macro_rules! block {
($($tt:tt)*) => { ... };
}Expand description
Entirely block some type from appearing in the generated
code. This can be useful if there is a type which is not
understood by bindgen or autocxx, and incorrect code is
otherwise generated.
This is ‘greedy’ in the sense that any functions/methods
which take or return such a type will also be blocked.
See also opaque.
A directive to be included inside include_cpp - see include_cpp for general information.